SmalltalkServer subclass: #RemoteActionAgent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Distributed-Computation'!
RemoteActionAgent comment:
'A RemoteActionAgent (RAA) is an entity that allows for the remote evaluation of Smalltalk code. 
An RAA is started whenever we need to perform remote operations on another Smalltalk image.
A single instance of this class should exist. Other instances will attempt to use a port in use, which 
will fail.'!


!RemoteActionAgent methodsFor: 'sending remote requests'!

send: anObject to: aHostname 
	"This sends a message to a remote host which is performed by the 
	remote host. A connection is opened, then closed after the object is 
	sent. Note that we do not post exceptions"

	self
		send: anObject
		to: aHostname
		handler: [:exception | exception return]!

send: anObject to: aHostname handler: aBlock
	"This sends a message to a remote host which is performed by the 
	remote host. A connection is opened, then closed after the object is 
	sent. Note that we have a user-provided exception handler"

	| anInterface |
	
	[anInterface := SmalltalkClient newAtHost: aHostname port: self class port handler: aBlock.
	anInterface notNil ifTrue: [anInterface nextPut: anObject]]
		valueNowOrOnUnwindDo: [anInterface notNil ifTrue: [anInterface close]]! !

!RemoteActionAgent methodsFor: 'remote action processing'!

remoteRequest: anObject 
	"Process the message just received"
	^(Compiler new)
		evaluate: (ReadStream on: anObject)
		in: nil
		receiver: self
		notifying: nil
		ifFail: []! !

!RemoteActionAgent methodsFor: 'initialize-release'!

initialize
	super initialize.
	self application: self.
	self callback: #remoteRequest:!

release
	self close.
	ObjectMemory garbageCollect.! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

RemoteActionAgent class
	instanceVariableNames: ''!


!RemoteActionAgent class methodsFor: 'class initialization'!

initialize
	"RemoteActionAgent initialize"

	| raa |
	raa := Smalltalk at: #RAA ifAbsent: [nil].
	(raa isKindOf: self)
		ifTrue: [raa release].
	Smalltalk at: #RAA put: self start! !

!RemoteActionAgent class methodsFor: 'class constants'!

port
	"Default port used to remote method calls"
	^9893! !

!RemoteActionAgent class methodsFor: 'inspection'!

peek
	"RemoteActionAgent peek"
	RemoteActionAgent allInstances inspect.! !

!RemoteActionAgent class methodsFor: 'initialize-release'!

release
	"RemoteActionAgent release"
	RAA release.
	Smalltalk removeKey: #RAA ifAbsent: [^self].
	ObjectMemory garbageCollect.!

start
	"RemoteActionAgent start"
	^self startAtPort: self port.!

startAtPort: anInteger
	^self newAtPort: anInteger! !

!RemoteActionAgent class methodsFor: 'cleanup'!

cleanup
	"RemoteActionObject cleanup"

	Object errorSignal handle: [:exception | exception return]
		do: [self allInstances do: [:anRAA | anRAA release]]! !
RemoteActionAgent initialize!


