GASTEUI subclass: #GeneticObjectDistributor
	instanceVariableNames: 'percentageExchangeHolder exchangeMultiplierHolder agent clientListHolder clientInputFieldHolder lastDistribution runProcess distributorLogHolder remoteHost '
	classVariableNames: 'DefaultExchangeMultiplier DefaultPercentageExchange '
	poolDictionaries: ''
	category: 'GADist'!


!GeneticObjectDistributor methodsFor: 'updating'!

changeRequest
	"Terminate this and all remote agents"

	super changeRequest.
	Cursor wait
		showWhile: 
			[self stopRemoteAgent.
			agent release].
	^true!

clientSelectionChanged
	"Make sure the input field mirrors the state of the client list latest selection"

	self clientInputFieldHolder value: (self clientListHolder selections isEmpty
			ifTrue: ['']
			ifFalse: [self clientListHolder selections at: self clientListHolder selections size])!

exchangeMultiplierChanged
	"Distribute the value to the remote hosts"

	self class defaultExchangeMultiplier: self exchangeMultiplierHolder value.
	self doRemoteAction: 'self exchangeMultiplierHolder setValue: ' , self exchangeMultiplierHolder value printString!

percentageExchangeChanged
	"Distribute the value to the remote hosts"

	self class defaultPercentageExchange: self percentageExchangeHolder value.
	self doRemoteAction: 'self percentageExchangeHolder setValue: ' , self percentageExchangeHolder value printString!

update: anAspect with: anObject from: aSender 
	"Exchange Genetic Programs on output, if appropriate. We exchange 
	programs at the end of a run in order to ensure that remote experiments 
	cause their final solutions to be sent to the master workstation"

	super
		update: anAspect
		with: anObject
		from: aSender.
	anAspect == #output
		ifTrue: 
			[self checkForObjectDistribution.
			aSender continue].
	anAspect == #endOfRun
		ifTrue: 
			[lastDistribution := self experiment evaluations.
			self distributeGeneticPrograms].
	anAspect == #experiment
		ifTrue: 
			[self stopExperiment.
			self experiment: anObject.
			lastDistribution := 0]! !

!GeneticObjectDistributor methodsFor: 'aspects'!

clientInputFieldHolder
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^clientInputFieldHolder isNil
		ifTrue:
			[clientInputFieldHolder := String new asValue]
		ifFalse:
			[clientInputFieldHolder]!

clientInputFieldHolder: aString
	^clientInputFieldHolder setValue: aString!

clientListHolder
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^clientListHolder isNil
		ifTrue:
			[clientListHolder := MultiSelectionInList new]
		ifFalse:
			[clientListHolder]!

clientListHolder: anOrderedCollection
	^clientListHolder list: anOrderedCollection!

distributorLogHolder
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^distributorLogHolder isNil
		ifTrue:
			[distributorLogHolder := String new asValue]
		ifFalse:
			[distributorLogHolder]!

distributorLogHolder: aString
	^distributorLogHolder setValue: aString!

exchangeMultiplierHolder
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^exchangeMultiplierHolder isNil
		ifTrue:
			[exchangeMultiplierHolder := self class defaultExchangeMultiplier asValue]
		ifFalse:
			[exchangeMultiplierHolder]!

exchangeMultiplierHolder: anInteger
	^exchangeMultiplierHolder setValue: anInteger!

percentageExchangeHolder
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^percentageExchangeHolder isNil
		ifTrue:
			[percentageExchangeHolder := self class defaultPercentageExchange asValue]
		ifFalse:
			[percentageExchangeHolder]!

percentageExchangeHolder: anInteger
	^percentageExchangeHolder setValue: anInteger! !

!GeneticObjectDistributor methodsFor: 'accessing'!

agent
	^agent!

lastDistribution: anInteger
	^lastDistribution := anInteger.!

remoteHost
	^remoteHost!

remoteHost: aString
	^remoteHost := aString.!

runProcess
	^runProcess!

runProcess: aProcess
	^runProcess :=aProcess! !

!GeneticObjectDistributor methodsFor: 'private'!

addToLog: aString 
	"Add new text to the log holder. If there's no builder it means that we 
	currently don't have a UI. It probably means that we're a remote 
	GeneticObjectDistributor"

	| textPane content difference |
	self builder isNil ifTrue: [^self].
	self distributorLogHolder value: self distributorLogHolder value , aString , '.\' withCRs.
	textPane := (self builder componentAt: #log) widget.
	content := textPane displayContents.	"See if the content height exceeds the pane size"
	difference := textPane bounds corner y - content height.
	difference < 0 ifTrue: [textPane scrollTo: 0 @ difference]!

checkForObjectDistribution
	"This method decides whether we should send off Genetic Programs to 
	our remote clients or not"

	| evaluations |
	evaluations := self experiment evaluations - lastDistribution.
	evaluations < 0
		ifTrue: 
			[evaluations := self experiment evaluations.
			lastDistribution := 0].
	evaluations >= self distributionFrequency
		ifTrue: 
			[lastDistribution := self experiment evaluations.
			self distributeGeneticPrograms]!

distributeGeneticPrograms
	"This distributes genetic programs to remote clients. We first ensure that 
	the collection of programs has had their fitness values computed"

	| aCollectionOfGeneticPrograms |
	aCollectionOfGeneticPrograms := self experiment getProportionOfPopulation: self percentageExchangeHolder value.
	aCollectionOfGeneticPrograms do: [:aGP | aGP fitnessEvaluation: self experiment].
	self doRemoteAction: aCollectionOfGeneticPrograms!

distributionFrequency
	^self  experiment outputFrequency * self exchangeMultiplierHolder value!

doRemoteAction: aString 
	"Perform a remote action"

	self clientListHolder selections do: [:selection | self agent
			send: aString
			to: selection
			handler: [:ex | self addToLog: selection , ': ' , ex errorString]]!

experimentHandler
	"This returns a block that is used to handle exception conditions"

	^
	[:exception | 
	self agent send: 'self addToLog: ' , ('Problem on ' , TransportInterface defaultConcreteClass getHostname , ': ' , exception errorString) printString to: self remoteHost.
	exception return]!

getRemoteActionAgent
	"Start the remote agents"

	| actionAgent  |
	actionAgent := RemoteActionAgent allInstances.
	^actionAgent isEmpty
		ifTrue: [(Dialog confirm: 'The Remote Action Agent (RAA) is not running on this workstation.\Do you want to start it?' withCRs)
						ifTrue: 
							[RemoteActionAgent initialize.
							RAA]
						ifFalse: [nil]]
		ifFalse: [actionAgent at: 1]!

initializeClientList
	"Obtain the list of clients to be used for distribution"
	self clientListHolder list: agent clients.!

loadFromFile: aString notifying: aHostname 
	| obj |
	^self
		protectedAction: 
			[obj := (Filename named: aString) fileIn.
			(obj isKindOf: GeneticAlgorithm)
				ifTrue: [self experiment: obj]
				ifFalse: [Object errorSignal raiseWith: obj]]
		handler: 
			[:exception | 
			self agent send: 'self addToLog: ' , ('Loading ' , aString  ,' on ', TransportInterface defaultConcreteClass getHostname, ': ' , exception errorString) printString to: aHostname.
			exception return]!

parameters: aString 
	"Respond to the request for the remote hosts experimental parameters"

	| parms aStream |
	aStream := WriteStream on: String new.
	self experiment storeOn: aStream.
	parms := aStream contents.
	self agent send: 'self remoteParametersAre: ' , parms , ' host: ' , TransportInterface defaultConcreteClass getHostname printString to: aString!

ping: aString 
	"Respond to a ping - write to the remote Distributor log"

	self agent send: 'self addToLog: ' , (self class name , ' on host ' , TransportInterface defaultConcreteClass getHostname , ' is responding') printString to: aString!

protectedAction: aBlock handler: anotherBlock 
	"This unconditionally protects a block from unhandled exceptions"

	(SignalCollection with: Object errorSignal with: OSErrorHolder errorSignal)
		handle: anotherBlock do: aBlock!

remoteParametersAre: aGeneticAlgorithm host: aString 
	"Display a dialog that gives us the parameters used on the remote 
	machine"

	| newExperiment |
	aGeneticAlgorithm isChanged: false.
	newExperiment := (GASTERemoteParameters openDialogWith: (GASTEUI dialog: aString experiment: aGeneticAlgorithm)) experiment.
	newExperiment isChanged ifTrue: [self agent send: newExperiment to: aString]!

setClients: anOrderedCollection
	self agent clients: anOrderedCollection.
	self initializeClientList.
	self selectAll.! !

!GeneticObjectDistributor methodsFor: 'initialize-release'!

initialize
	super initialize.
	lastDistribution := 0.
	agent := GeneticObjectDistributionAgent start.
	agent application: self.!

postBuildWith: aBuilder 
	"Initialize all the fields now that the agent has been created"

	super postBuildWith: aBuilder.
	self clientListHolder list: agent clients.
	self exchangeMultiplierHolder onChangeSend: #exchangeMultiplierChanged to: self.
	self percentageExchangeHolder onChangeSend: #percentageExchangeChanged to: self.
	self clientListHolder selectionIndexHolder onChangeSend: #clientSelectionChanged to: self.
	(aBuilder componentAt: #log) widget controller continuousAccept: true; model: self distributorLogHolder!

release
	self stopExperiment.
	self agent release.
	super release.
	ObjectMemory garbageCollect.! !

!GeneticObjectDistributor methodsFor: 'local actions'!

cleanupHost
	"Do a spring clean on the remote hosts"

	| raa |
	raa := self getRemoteActionAgent.
	self clientListHolder selections do: [:selection | raa send: 'GeneticObjectDistributor cleanup' to: selection]!

clickedAdd
	"This method is invoked when the add key is pressed"

	(self isValidName: self clientInputFieldHolder value) ifTrue: [
		agent addHost: self clientInputFieldHolder value.
		self initializeClientList.
	]
	ifFalse: [Dialog warn: 'Invalid name - ignored'].!

continueRemoteRun
	"Continue with experiments on the selected hosts"

	self doRemoteAction: 'self continueExperiment'!

deselectAll
	"Clear the list of selections"
	self clientListHolder clearAll.!

editParameters
	"This opens the Parameters dialog for editing"

	self doRemoteAction: 'self parameters: ', TransportInterface defaultConcreteClass getHostname printString!

exportHost
	"Ping the host to see whats there"

	|  aStream |
	self clientListHolder selections isEmpty
		ifFalse: 
			[aStream := WriteStream on: (String new: 100).
			(self clientListHolder selections copyWith: TransportInterface defaultConcreteClass getHostname) storeOn: aStream.
			self doRemoteAction: 'self setClients: ' , aStream contents]!

inspectGeneticObjectDistributionAgent
	"Inspect the agent"
	self agent inspect.!

inspectGeneticObjectDistributor
	"Inspect myself"
	self inspect.!

loadFile
	"Send a request to load a file containing an experiment. Note that we use 
	a protected action in order to get errors reported"

	| myHost filename |
	self clientListHolder selections isEmpty
		ifFalse: 
			[filename := Dialog requestFileName: 'Enter filename: ' default: 'GPEIST'.
			myHost := TransportInterface defaultConcreteClass getHostname printString.
			filename isEmpty ifFalse: [self doRemoteAction: 'self loadFromFile: ' , filename printString , ' notifying: ' , myHost]]!

pingHost
	"Ping the host to see whats there"

	| myName |
	myName := TransportInterface defaultConcreteClass getHostname.
	self doRemoteAction: 'self ping: ', myName printString!

removeHost
	"Remove the host from those linked into the experiment"

	self clientListHolder selections do: [:selection | self agent removeHost: selection].
	self initializeClientList!

selectAll
	"Select every host"
	self clientListHolder selectAll.!

startRemoteAgent
	"Start up selected hosts"

	| raa |
	raa := self getRemoteActionAgent.
	self clientListHolder selections do: [:selection | raa
			send: 'GeneticObjectDistributor exchange: ' , self exchangeMultiplierHolder value printString , ' percentage: ' , self percentageExchangeHolder value printString , ' remoteHost: ' , TransportInterface defaultConcreteClass getHostname printString
			to: selection
			handler: [:ex | self addToLog: selection , ': ' , ex errorString]]!

startRemoteRun
	"Start up experiments on the selected hosts"

	self doRemoteAction: 'self startExperiment'!

stopRemoteAgent
	"Terminate selected hosts"

	self doRemoteAction: 'self release'!

stopRemoteRun
	"Stop experiments on the selected hosts"

	self doRemoteAction: 'self stopExperiment'! !

!GeneticObjectDistributor methodsFor: 'remote actions'!

continueExperiment
	"Continue the experiment"

	self stopExperiment.
	self runProcess: ([self protectedAction: [self experiment continueExperiment] handler: self experimentHandler]
			forkAt: Processor systemRockBottomPriority)!

startExperiment
	"Start the experiment"

	self stopExperiment.
	self runProcess: ([self protectedAction: [self experiment runExperiment] handler: self experimentHandler]
			forkAt: Processor systemRockBottomPriority)!

stopExperiment
	"Stop the experiment"

	| tProcess |
	self runProcess notNil
		ifTrue: 
			[tProcess := self runProcess.
			self runProcess: nil.
			tProcess terminate]! !

!GeneticObjectDistributor methodsFor: 'remote action processing'!

remoteActionRequest: anObject 
	"Process the message just received; presumably a string"
	^(Compiler new)
		evaluate: (ReadStream on: anObject)
		in: nil
		receiver: self
		notifying: nil
		ifFail: []!

remoteRequest: anObject 
	"Process the received object. We can deal with individual 
	GeneticObjects, Arrays of GeneticObjects, whole experiments or 
	populations and strings that, when evaluated, cause remote actions to 
	occur"

	(anObject isKindOf: GeneticPopulationMember)
		ifTrue: 
			[self addToLog: anObject printString.
			^self experiment insertInPopulation: anObject].
	(anObject isKindOf: OrderedCollection)
		ifTrue: 
			[self addToLog: anObject printString.
			^self experiment insertArrayInPopulation: anObject].
	(anObject isKindOf: GeneticAlgorithm)
		ifTrue: [^self experiment: anObject].
	(anObject isKindOf: GeneticPopulation)
		ifTrue: [^self experiment population: anObject].
	(anObject isKindOf: String)
		ifTrue: [^self remoteActionRequest: anObject]! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

GeneticObjectDistributor class
	instanceVariableNames: ''!


!GeneticObjectDistributor class methodsFor: 'interface specs'!

windowSpec
	"UIPainter new openOnClass: self andSelector: #windowSpec"

	<resource: #canvas>
	^#(#FullSpec 
		#window: 
		#(#WindowSpec 
			#label: 'Genetic Program Distributor' 
			#min: #(#Point 323 480 ) 
			#bounds: #(#Rectangle 351 144 674 624 ) 
			#flags: 4 
			#menu: #windowMenu ) 
		#component: 
		#(#SpecCollection 
			#collection: #(
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.860681 0 0.482143 ) 
					#label: '10' ) 
				#(#SequenceViewSpec 
					#layout: #(#LayoutFrame 0 0.0340557 0 0.631696 0 0.934984 0 0.830357 ) 
					#model: #clientListHolder 
					#menu: #clientMenu 
					#multipleSelections: true ) 
				#(#InputFieldSpec 
					#layout: #(#LayoutFrame 0 0.213622 0 0.845982 0 0.931888 0 0.90625 ) 
					#model: #clientInputFieldHolder 
					#menu: #fieldMenu ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.0309598 0 0.845982 ) 
					#label: 'Client:' ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.0928793 0 0.919643 0 0.28483 0 0.982143 ) 
					#model: #clickedAdd 
					#label: 'Add' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.668731 0 0.917411 0 0.860681 0 0.979911 ) 
					#model: #clickedClose 
					#label: 'Close' 
					#isDefault: true 
					#defaultable: true ) 
				#(#TextEditorSpec 
					#layout: #(#LayoutFrame 0 0.0216718 0 0.0669643 0 0.922601 0 0.292411 ) 
					#name: #log 
					#model: #distributorLogHolder 
					#menu: #fieldMenu 
					#isReadOnly: false ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.0309598 0 0.575893 ) 
					#label: 'Clients' ) 
				#(#SliderSpec 
					#layout: #(#LayoutFrame 0 0.0247678 0 0.308036 0 0.873065 0 0.350446 ) 
					#model: #percentageExchangeHolder 
					#orientation: #horizontal 
					#start: 0 
					#stop: 100 
					#step: 1 ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.00928792 0 0.350446 ) 
					#label: '0%' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.820433 0 0.350446 ) 
					#label: '100%' ) 
				#(#InputFieldSpec 
					#layout: #(#LayoutFrame 0 0.526316 0 0.363839 0 0.690402 0 0.424107 ) 
					#model: #percentageExchangeHolder 
					#tabable: false 
					#alignment: #center 
					#isReadOnly: true 
					#type: #number ) 
				#(#SliderSpec 
					#layout: #(#LayoutFrame 0 0.0247678 0 0.448661 0 0.873065 0 0.491071 ) 
					#model: #exchangeMultiplierHolder 
					#orientation: #horizontal 
					#start: 1 
					#stop: 10 
					#step: 1 ) 
				#(#InputFieldSpec 
					#layout: #(#LayoutFrame 0 0.526316 0 0.508929 0 0.690402 0 0.569196 ) 
					#model: #exchangeMultiplierHolder 
					#tabable: false 
					#alignment: #center 
					#isReadOnly: true 
					#type: #number ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.00619195 0 0.491071 ) 
					#label: '1' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.114551 0 0.508929 ) 
					#label: 'Exchange Freq:' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.185759 0 0.363839 ) 
					#label: 'Exchange %:' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.0185759 0 0.00669643 ) 
					#label: 'Log' ) ) ) )! !

!GeneticObjectDistributor class methodsFor: 'resources'!

clientMenu
	"UIMenuEditor new openOnClass: self andSelector: #clientMenu"

	<resource: #menu>
	^#(#Menu #(
			#(#MenuItem 
				#label: 'Start remote agent' 
				#accessCharacterPosition: 14 ) 
			#(#MenuItem 
				#label: 'Stop remote agent' 
				#accessCharacterPosition: 6 ) 
			#(#MenuItem 
				#label: 'Start remote run' 
				#accessCharacterPosition: 1 ) 
			#(#MenuItem 
				#label: 'Stop remote run' 
				#accessCharacterPosition: 2 ) 
			#(#MenuItem 
				#label: 'Continue remote run' 
				#accessCharacterPosition: 1 ) 
			#(#MenuItem 
				#label: 'Select all' ) 
			#(#MenuItem 
				#label: 'Deselect all' ) 
			#(#MenuItem 
				#label: 'Edit parameters' 
				#accessCharacterPosition: 1 ) 
			#(#MenuItem 
				#label: 'Load file' 
				#accessCharacterPosition: 1 ) 
			#(#MenuItem 
				#label: 'Remove host' 
				#accessCharacterPosition: 1 ) 
			#(#MenuItem 
				#label: 'Cleanup host' 
				#accessCharacterPosition: 2 ) 
			#(#MenuItem 
				#label: 'Ping host' 
				#accessCharacterPosition: 1 ) 
			#(#MenuItem 
				#label: 'Export host' 
				#accessCharacterPosition: 1 ) ) #(2 3 2 2 4 ) #(#startRemoteAgent #stopRemoteAgent #startRemoteRun #stopRemoteRun #continueRemoteRun #selectAll #deselectAll #editParameters #loadFile #removeHost #cleanupHost #pingHost #exportHost ) ) decodeAsLiteralArray!

windowMenu
	"UIMenuEditor new openOnClass: self andSelector: #windowMenu"

	<resource: #menu>
	^#(#Menu #(
			#(#MenuItem 
				#label: 'Inspect' 
				#accessCharacterPosition: 1 
				#submenu: #(#Menu #(
						#(#MenuItem 
							#label: 'Distributor' 
							#accessCharacterPosition: 1 ) 
						#(#MenuItem 
							#label: 'Agent' 
							#accessCharacterPosition: 1 ) ) #(2 ) #(#inspectGeneticObjectDistributor #inspectGeneticObjectDistributionAgent ) ) ) ) #(1 ) #(nil ) ) decodeAsLiteralArray! !

!GeneticObjectDistributor class methodsFor: 'class initialization'!

initialize
	"GeneticObjectDistributor initialize"
	self defaultExchangeMultiplier: 1.
	self defaultPercentageExchange: 1.! !

!GeneticObjectDistributor class methodsFor: 'accessing'!

defaultExchangeMultiplier
	^DefaultExchangeMultiplier!

defaultExchangeMultiplier: anInteger
	^DefaultExchangeMultiplier := anInteger!

defaultPercentageExchange
	^ DefaultPercentageExchange!

defaultPercentageExchange: anInteger
	^ DefaultPercentageExchange  := anInteger! !

!GeneticObjectDistributor class methodsFor: 'cleanup'!

cleanup
	"GeneticObjectDistributor cleanup"

	[Object errorSignal handle: [:ex | ex return]
		do: [self allInstances do: [:aGOD | aGOD release]]]
		valueNowOrOnUnwindDo: [ObjectMemory garbageCollect]! !

!GeneticObjectDistributor class methodsFor: 'instance creation'!

exchange: anInteger percentage: anotherInteger
	"Create a new GeneticObjectDistributor"
	^self exchange: anInteger percentage: anotherInteger remoteHost: nil.!

exchange: anInteger percentage: anotherInteger remoteHost: aString
	"Create a new GeneticObjectDistributor"
	self defaultExchangeMultiplier: anInteger.
	self defaultPercentageExchange: anotherInteger.
	^(self new) remoteHost: aString; yourself.! !

RemoteActionAgent subclass: #GeneticObjectDistributionAgent
	instanceVariableNames: 'clients '
	classVariableNames: 'DefaultClients '
	poolDictionaries: ''
	category: 'GADist'!


!GeneticObjectDistributionAgent methodsFor: 'initialize-release'!

initialize
	super initialize.
	clients := self class defaultClients.!

release
	"Release the port and stop any experiment running. Save the client list in 
	order that we can reuse it later"

	super release.
	self class defaultClients: self clients.! !

!GeneticObjectDistributionAgent methodsFor: 'accessing'!

addHost: aString
	(clients includes: aString) ifFalse: [clients add: aString].
	^aString.!

clients
	^clients.!

clients: anOrderedCollection 
	"Only store client names which are not my host"

	| myName |
	myName := TransportInterface defaultConcreteClass getHostname.
	^clients := anOrderedCollection select: [:name | name ~= myName]!

removeHost: aString
	^clients remove: aString ifAbsent: ["don't care"]! !

!GeneticObjectDistributionAgent methodsFor: 'private'!

doRemoteAction: aString
	"Perform a remote action"

	self clients do: [:aHost | self send: aString to: aHost]! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

GeneticObjectDistributionAgent class
	instanceVariableNames: ''!


!GeneticObjectDistributionAgent class methodsFor: 'class initialization'!

initialize
	"GeneticObjectDistributionAgent initialize"
	self defaultClients: OrderedCollection new.! !

!GeneticObjectDistributionAgent class methodsFor: 'class constants'!

port
	^9894! !

!GeneticObjectDistributionAgent class methodsFor: 'accessing'!

defaultClients
	^DefaultClients.!

defaultClients: anOrderedCollection
	^DefaultClients := anOrderedCollection.! !

!GeneticObjectDistributionAgent class methodsFor: 'cleanup'!

cleanup
	"GeneticObjectDistributionAgent cleanup"

	Object errorSignal handle: [:exception | exception return]
		do: [self allInstances do: [:aGODA | aGODA release]]! !
GeneticObjectDistributor initialize!

GeneticObjectDistributionAgent initialize!


